/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is from Linky
 *
 * The Initial Developer of the Original Code is Henrik Gemal.
 * Portions created by the Initial Developer are Copyright (C) 2002-2003
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Henrik Gemal <linky@gemal.dk> http://gemal.dk
 *   David Costanzo <david_costanzo at yahoo dot com>, modified for Dict
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

// product related settings
// you can change these
const myProductName            = "Dict";
const myProductRegKey          = "dict";
const myProductRegVersion      = "0.6.74";
const myProductRegVersionBuild = "20080704";
const myJarFileName  = "dict.jar";
const myJarFileSize  = 45;  // the filesize in KB of myJarFileName
const myLocales      = new Array("en-US"); // the locales available
const myJarCheckDupe = 1; // check if myJarFileName exists in both 
                          // personal chrome folder and global chrome folder

// global settings; don't change these
const sysChromeGlobal     = getFolder("Chrome");
const sysChromeGlobalJar  = getFolder("Chrome", myJarFileName);
const sysComponentsGlobal = getFolder("Components");

const sysChromeUser       = getFolder("Current User", "chrome");
const sysChromeUserJar    = getFolder("Current User", "chrome/" + myJarFileName);
const sysComponentsUser   = getFolder("Current User", "Components");

const INSTALL_METHOD_Cancelled = "cancelled";
const INSTALL_METHOD_Global    = "global";
const INSTALL_METHOD_Personal  = "user";

// verify disk space in kilobytes
function verifyDiskSpace(dirPath, spaceRequired) 
{
    var spaceAvailable = fileGetDiskSpaceAvailable(dirPath);
    if (parseInt(spaceAvailable/1024) < spaceRequired) {
        logComment(
            "Insufficient disk space: " + dirPath + "\n" + 
            "Required : " + spaceRequired + " KB\n"    + 
            "Available: " + spaceAvailable + " KB");
        return false;
    }
    else {
        return true;
    }
}


// install_method:
//   0 means that the installation was cancelled.
//   1 is in the global chrome folder. This *has* to be the default!
//   2 in in the personal profile folder. This is only supported for newer builds.

function getInstallMethod() 
{

    if (myJarCheckDupe && File.exists(sysChromeGlobalJar) && 
        File.exists(sysChromeUserJar)) {

        alert(
            myProductName + " is installed in BOTH the personal chrome folder:\n" + 
            sysChromeUserJar + "\n" + 
            "and the global chrome folder:\n" + 
            sysChromeGlobalJar + "\n\n" + 
            "This will cause weird behavior. Please delete one of the instances!");
        return INSTALL_METHOD_Cancelled;
    }

    // Early version of Mozilla didn't have personal profiles.
    // Assumes that anyone without a buildID knowns what they are doing.
    if (!buildID || buildID < 2003030700) {

        ans = confirm(
            "Click OK to install " + myProductName + " " + myProductRegVersion + " in the global chrome folder:\n" + 
            sysChromeGlobal);
        if (ans) {
            return INSTALL_METHOD_Global;
        }

        alert(
            "You didn't choose any of the supported installation options!\n" + 
            "Please restart the installation!");
        return INSTALL_METHOD_Cancelled;
    }


    alert(
       "You can install " + myProductName + " " + myProductRegVersion + " in two ways:\n\n" +
       "1) in the personal profile folder. This makes " + myProductName + " available only in the current profile.\n" + 
       "2) in the global chrome folder. This makes " + myProductName + " available in all profiles.\n" +
       "\n" +
       "You will now be prompted to select a way...");

    var ans;
        
    // Check if extension already is installed
    if (File.exists(sysChromeGlobalJar)) {

        ans = confirm(
            "A version of " + myProductName + " is already installed in the global chrome folder:\n" + 
            sysChromeGlobal + "\n" +
            "Click OK to install " + myProductName + " into this folder.\n" + 
            "(Clicking Cancel will cancel the installation).");
        if (ans) {
            return INSTALL_METHOD_Global;
        }
        else {
            return INSTALL_METHOD_Cancelled;
        }
    } 
    else if (File.exists(sysChromeUserJar)) {

        ans = confirm(
            "A version of " + myProductName + " is already installed in the personal chrome folder:\n" + 
            sysChromeUser + "\n" + 
            "Click OK to install " + myProductName + " into this folder.\n" + 
            "(Clicking Cancel will cancel the installation).");
        if (ans) {
            return INSTALL_METHOD_Personal;
        }
        else {
            return INSTALL_METHOD_Cancelled;
        }
    }
        
    ans = confirm("Click OK to install in your personal profile folder:\n" + sysChromeUser);
    if (ans) {
        return INSTALL_METHOD_Personal;
    }

    ans = confirm("Click OK to install in the global chrome folder:\n" + sysChromeGlobal);
    if (ans) {
        return INSTALL_METHOD_Global;
    }

    alert(
        "You didn't choose any of the supported installation options!\n" +
        "Please restart the installation!");

    return INSTALL_METHOD_Cancelled;
}



var install_method = getInstallMethod();

// did the user cancel?
if (install_method == INSTALL_METHOD_Cancelled) {
    // is this necessary?
    cancelInstall();
}
else {

    // select the folder to into into
    var chromeFolder;
    var chromeJarFolder;
    var componentsFolder;
    var contentFlags;
    var localeFlags;

    if (install_method == INSTALL_METHOD_Global) {
        chromeFolder     = sysChromeGlobal;
        chromeJarFolder  = sysChromeGlobalJar;
        componentsFolder = sysComponentsGlobal;
        contentFlags     = CONTENT | DELAYED_CHROME;
        localeFlags      = LOCALE  | DELAYED_CHROME;
    }
    else {
        chromeFolder     = sysChromeUser;
        chromeJarFolder  = sysChromeUserJar;
        componentsFolder = sysComponentsUser;
        contentFlags     = CONTENT | PROFILE_CHROME;
        localeFlags      = LOCALE  | PROFILE_CHROME;
    }
 
    var err;

    if (buildID) {
        logComment(myProductName + " version " + myProductRegVersion + " being installed on " + buildID);
    }
    else {
        logComment(myProductName + " version " + myProductRegVersion + " being installed on a build without buildID");
    }
    
    logComment("Installation method is: " + install_method);

    // init the installation
    err = initInstall(
        myProductName, 
        myProductRegKey, 
        myProductRegVersion);

    setPackageFolder(chromeFolder);

    // check if there's disk space for myJarFileSize 	 
    if (!err) {
        if (!verifyDiskSpace(chromeFolder, myJarFileSize)) {
            err = INSUFFICIENT_DISK_SPACE;
        }
    }

    // add the protocol provider
    if (!err) {

        err = addFile(
            myProductName,
            myProductRegVersion,
            'components/dict-service.js', 
            componentsFolder,
            'dict-service.js');

        if (err) {
            logComment("Problem adding protocol handler. Error code: " + err);
        }
        else {
            logComment("ok adding protocol handler file.");
        }
    }

    // add the jar file
    if (!err) {

        err = addFile(
            myProductName,
            "chrome/" + myJarFileName,
            chromeFolder,
            "");
        
        if (err) {
            logComment("Problem adding jar file. Error code: " + err);
        }
        else {
            logComment("ok adding jar file.");
        }
    }

    // register the content
    if (!err) {

        err = registerChrome(
            contentFlags,
            chromeJarFolder,
            "content/" + myProductRegKey + "/");

        if (err) {
            logComment("Problem registering the content chrome. Error code: " + err);
        } 
        else {
            logComment("ok registering the content chrome.");
        }
    }

    // register the locale
    if (!err) {

        for (var i = 0; i < myLocales.length; i++) {
            
            var relativeFolder = "locale/" + myLocales[i] + "/" + myProductRegKey + "/";
            
            err = registerChrome(
                localeFlags,
                chromeJarFolder,
                relativeFolder);
            if (err) {
                logComment("Problem registering locale at: " + relativeFolder);
                break;
            }
        }

        if (err) {
            logComment("Problem registering the locale chrome. Error code: " + err);
        }
        else {
            logComment("ok registering the locale chrome.");
        }
    }

    // do the install
    if (!err) {
        err = performInstall();
        if (0 <= err) {
            // success
            alert(
                myProductName + " version " + myProductRegVersion + " has been successfully installed!\n\n" + 
                "Please restart your browser to complete installation.");

            err = 0;
        }
        else {
            // report the error

            var errorMessage = 
                myProductName + " version " + myProductRegVersion + " could not be installed!\n" + 
                "Error code: " + err + "\n\n";

            // add more information to the error message - as appropriate
            if (err == -239                               &&
                install_method == INSTALL_METHOD_Personal &&
                File.exists(sysChromeUserJar)) {

                // Generic Chrome Registry Error 
                // see http://bugzilla.mozdev.org/show_bug.cgi?id=5914 for details

                errorMessage += 
                    "You may be able to successfully upgrade the Dict extension by restarting\n" +
                    "Mozilla and retrying the upgrade as the first thing you do.\n" +
                    "If this doesn't work, you must remove the Dict extension from the chrome\n" +
                    "registry before trying to install.\n";
            }

            alert(errorMessage);
            logComment("Problem performing install. Error code: " + err);
        }
    }
    else {
        // there was a problem before we even got to try performInstall()
        logComment("Problem installing. Error code: " + err);

        var errorMessage = 
            myProductName + " version " + myProductRegVersion + " could not be installed!\n" + 
            "Error code: " + err + "\n\n";
        
        if (err == -202 &&
            install_method == INSTALL_METHOD_Global) {
            
            errorMessage += 
                "You do not have sufficient permissions to install into the global chrome folder.\n" + 
                "Try installing into your personal chrome folder, instead.\n";
        }

        alert(errorMessage);
        logComment("Problem performing install. Error code: " + err);
    }

    if (err) {
        cancelInstall(err);
    }
}
